import { Injectable, Inject, Directive, ElementRef, Renderer2, Input, ContentChildren, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { ScrollingModule } from '@angular/cdk/scrolling';
import { fromEvent, BehaviorSubject } from 'rxjs';
import { debounceTime, withLatestFrom } from 'rxjs/operators';
import { coerceBooleanProperty } from '@angular/cdk/coercion';

// scrollspy animations
const SCROLLSPY_ANIMATION_SMOOTH = 'smooth';
const SCROLLSPY_ANIMATION_INSTANT = 'instant';
const SCROLLSPY_ANIMATION_AUTO = 'auto';

class MccScrollspyService {
    constructor(window) {
        this.window = window;
        /**
         * When scroll is from click event, change this attr to true
         * So scroll event obeservable doesn't emit any update
         */
        this._fromClick = false;
        /**
         * List of scrollspy group
         */
        this.data = [];
        // listen to scroll event
        this._scrollSub = fromEvent(window, 'scroll')
            .pipe(debounceTime(50), withLatestFrom(() => window.scrollY))
            .subscribe(position => {
            if (!this._fromClick) {
                this._updateFocused(position);
            }
            this._fromClick = false;
        });
    }
    ngOnDestroy() {
        if (this._scrollSub && !this._scrollSub.closed) {
            this._scrollSub.unsubscribe();
        }
    }
    /**
     * Update information about wich element is on focus
     * @param position number
     */
    _updateFocused(position) {
        this.data.forEach(group => {
            const items = [];
            group.items.getValue().forEach((item, index) => {
                item.focus = false;
                if (item.top <= position) {
                    if (items[index - 1]) {
                        items[index - 1].focus = false;
                    }
                    item.focus = true;
                }
                items.push(item);
            });
            group.items.next(items);
        });
    }
    /**
     * Create new group of items
     * @param name string
     * @param items MccScrollspyItemDirective[]
     * @param animation ScrollBehavior
     */
    create(name, items, animation) {
        let group = this.data.find(group => group.name === name);
        if (!group) {
            group = {
                name: name,
                animation: animation || SCROLLSPY_ANIMATION_SMOOTH,
                items: new BehaviorSubject(items || []),
            };
            this.data.push(group);
        }
        else {
            group.items.next(items || []);
        }
        return group;
    }
    /**
     * Return observable of the group
     * @param name string
     */
    group(name) {
        let group = this.data.find(g => g.name === name);
        if (!group) {
            group = this.create(name);
        }
        return group.items.asObservable();
    }
    /**
     * Scroll to one of the items
     * @param name string
     * @param id string
     */
    scrollTo(name, id) {
        const group = this.data.find(group => group.name === name);
        group.items.getValue().forEach(item => {
            item.focus = false;
            if (item.id === id) {
                this._fromClick = true;
                this._updateFocused(item.top);
                window.scrollTo({ top: item.top, behavior: group.animation });
            }
        });
    }
}
MccScrollspyService.decorators = [
    { type: Injectable }
];
MccScrollspyService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: ['Window',] }] }
];

class MccScrollspyItemDirective {
    constructor(elementRef, renderer) {
        this.elementRef = elementRef;
        this.renderer = renderer;
    }
    /**
     * Hold the element id, if element doesn't have id
     * the method will create one
     */
    set id(id) {
        if (!id) {
            id = this._createId();
            this.renderer.setProperty(this.elementRef.nativeElement, 'id', id);
        }
        this._id = id;
    }
    get id() {
        return this._id;
    }
    /**
     * Element distance of the top
     */
    get top() {
        return this.elementRef.nativeElement.offsetTop;
    }
    /**
     * Element is focused
     */
    set focus(focused) {
        this._focused = coerceBooleanProperty(focused);
    }
    get focus() {
        return this._focused;
    }
    ngAfterContentInit() {
        if (!this.label) {
            this.label = this.elementRef.nativeElement.textContent;
        }
        this.id = this.elementRef.nativeElement.id;
    }
    /**
     * Create an ID for the element
     */
    _createId() {
        let tmpID = this.label.toLowerCase().replace(/[ ]+/gi, '_');
        return `mcc_scrollspy_${tmpID}`;
    }
}
MccScrollspyItemDirective.decorators = [
    { type: Directive, args: [{
                selector: '[mccScrollspyItem], [mcc-scrollspy-item]',
                exportAs: 'mccScrollspyItem',
            },] }
];
MccScrollspyItemDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
MccScrollspyItemDirective.propDecorators = {
    focus: [{ type: Input, args: ['focus',] }],
    label: [{ type: Input }]
};
class MccScrollspyGroupDirective {
    constructor(mccScrollspyService) {
        this.mccScrollspyService = mccScrollspyService;
    }
    /**
     * Name of the scrollspy group
     */
    set name(name) {
        this._name = name;
    }
    ngAfterContentInit() {
        // add the group items
        const items = this.items.map(item => item);
        this.mccScrollspyService.create(this._name, items);
    }
}
MccScrollspyGroupDirective.decorators = [
    { type: Directive, args: [{
                selector: '[mccScrollspyGroup], [mcc-scrollspy-group]',
                exportAs: 'mccScrollspyGroup',
            },] }
];
MccScrollspyGroupDirective.ctorParameters = () => [
    { type: MccScrollspyService }
];
MccScrollspyGroupDirective.propDecorators = {
    items: [{ type: ContentChildren, args: [MccScrollspyItemDirective,] }],
    name: [{ type: Input, args: ['mccScrollspyGroup',] }]
};

const ɵ0 = window;
class MccScrollspyModule {
}
MccScrollspyModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, ScrollingModule],
                providers: [MccScrollspyService, { provide: 'Window', useValue: ɵ0 }],
                declarations: [MccScrollspyGroupDirective, MccScrollspyItemDirective],
                exports: [MccScrollspyGroupDirective, MccScrollspyItemDirective],
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { MccScrollspyGroupDirective, MccScrollspyItemDirective, MccScrollspyModule, MccScrollspyService, SCROLLSPY_ANIMATION_AUTO, SCROLLSPY_ANIMATION_INSTANT, SCROLLSPY_ANIMATION_SMOOTH, ɵ0 };
//# sourceMappingURL=material-community-components-scrollspy.js.map
