import { EventEmitter, Component, ChangeDetectionStrategy, ChangeDetectorRef, Input, Output, Directive, forwardRef, ElementRef, Renderer2, HostListener, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { PortalModule } from '@angular/cdk/portal';
import { OverlayModule } from '@angular/cdk/overlay';
import { MatButtonModule } from '@angular/material/button';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { BehaviorSubject } from 'rxjs';
import { NG_VALUE_ACCESSOR } from '@angular/forms';

/**
 * contants to create timer with HOURS or MINUTES
 */
const HOURS = ['12', '11', '1', '10', '2', '9', '3', '8', '4', '7', '5', '6'];
const HOURS24 = ['00', '23', '13', '22', '14', '21', '15', '20', '16', '19', '17', '18'];
const MINUTES = [
    '00',
    '55',
    '05',
    '50',
    '10',
    '45',
    '15',
    '40',
    '20',
    '35',
    '25',
    '30',
];

class MccTimerPickerComponent {
    constructor(changeDetectorRef) {
        this.changeDetectorRef = changeDetectorRef;
        /**
         * Receive selected _hour after confirm
         */
        this._selectedHour = '12';
        /**
         * Receive selected _minute after confirm
         */
        this._selectedMinute = '00';
        /**
         * Receive selected _period after confirm
         */
        this._selectedPeriod = 'am';
        this._clock = new BehaviorSubject(HOURS);
        this._focus = 'hour';
        this._hour = '12';
        this._minute = '00';
        this._period = 'am';
        this._hideButtons = false;
        /**
         * Format of the hour to be emited on confirm
         */
        this.format = '12';
        this.min = '00:00 am';
        this.max = '12:00 pm';
        /**
         * Change btnCancel label
         */
        this.btnCancel = 'Cancel';
        /**
         * Change btnConfirm label
         */
        this.btnConfirm = 'Confirm';
        /**
         * Event emited when confirm button is pressed.
         * If buttons are hidden, the event is emited when value is changed
         */
        this.selected = new EventEmitter();
        /**
         * Set to true when timer picker have been connected with another component
         */
        this.connected = false;
    }
    /**
     * Current value (hour/minute) to create the clock
     */
    get clock$() {
        return this._clock.asObservable();
    }
    /**
     * Return hours in 24h format
     */
    get hours24() {
        return HOURS24;
    }
    /**
     * Type there is in focus (hour/minute)
     */
    get focus() {
        return this._focus;
    }
    set focus(value) {
        if (value !== this._focus) {
            this._focus = value;
            this._clock.next(this._focus === 'hour' ? HOURS : MINUTES);
        }
    }
    /**
     * State of the overlay
     */
    get isOpen() {
        return this._isOpen;
    }
    set isOpen(value) {
        this._isOpen = coerceBooleanProperty(value);
        this.changeDetectorRef.detectChanges();
    }
    /**
     * Return temporary selected hour (const HOURS)
     */
    get hour() {
        return this._hour;
    }
    /**
     * Checks if hour has only one digit
     */
    get needsSpacer() {
        return parseInt(this._hour, 10) < 10;
    }
    /**
     * Return temporary selected minute (const MINUTES)
     */
    get minute() {
        return this._minute;
    }
    /**
     * Return temporary selected period (am/pm)
     */
    get period() {
        return this._period;
    }
    /**
     * Hide Confirm and Cancel buttons
     */
    set hideButtons(value) {
        this._hideButtons = coerceBooleanProperty(value);
    }
    get hideButtons() {
        return this._hideButtons;
    }
    /**
     * Return timer option class to create line between the middle of the clock and
     * the option
     */
    getSelectedClass() {
        let name = 'selected-index-';
        if (this.focus === 'hour') {
            // convert to number
            const hour = parseInt(this.hour, 10);
            // check if hour is mid night or afternoon in 24h format
            if (hour === 0 || hour > 12) {
                name += HOURS24.findIndex(h => h === this.hour);
                name += ' hour-format-24';
            }
            else {
                name += HOURS.findIndex(h => h === this.hour);
            }
        }
        else {
            name += MINUTES.findIndex(m => m === this.minute);
        }
        return name;
    }
    /**
     * Select option from the clock.
     */
    select(value) {
        if (this.focus === 'hour') {
            this._hour = value;
            this.focus = 'min';
        }
        else {
            this._minute = value;
            // if buttons are hidden, emit new event when value is changed
            if (this._hideButtons) {
                this.confirmSelectedTime();
            }
        }
    }
    /**
     * Returns array containing time, hour and period fragments from time string
     */
    parseTimeInput(time) {
        const parsed = time.split(/\s|:/).map((fragment, index) => {
            return index === 2 ? fragment : parseInt(fragment, 10);
        });
        if (parsed.length === 2) {
            // assume we are using 24 hour time format
            const hours = parsed[0];
            if (hours > 11) {
                parsed[0] = hours - 12;
                parsed.push('pm');
            }
            else {
                parsed.push('am');
            }
        }
        return parsed;
    }
    /**
     * Returns true if option value is not valid
     */
    isOptionDisabled(value) {
        const [minHour, minMinutes, minPeriod] = this.parseTimeInput(this.min);
        const [maxHour, maxMinutes, maxPeriod] = this.parseTimeInput(this.max);
        const optionValue = parseInt(value, 10);
        const selectedHour = parseInt(this._hour, 10);
        const selectedPeriod = this._period;
        if (this.focus === 'hour') {
            if (optionValue < minHour && selectedPeriod === minPeriod) {
                return true;
            }
            else if (optionValue > maxHour && selectedPeriod === maxPeriod) {
                return true;
            }
        }
        else {
            if (selectedHour === minHour && selectedPeriod === minPeriod && optionValue < minMinutes) {
                return true;
            }
            else if (selectedHour === maxHour && selectedPeriod === maxPeriod && optionValue > maxMinutes) {
                return true;
            }
        }
        return false;
    }
    /**
     * Change period of the clock
     */
    changePeriod(period) {
        this._period = period;
        // if buttons are hidden, emit new event when value is changed
    }
    /**
     * Update selected color, close the panel and notify the user
     */
    backdropClick() {
        if (this._hideButtons) {
            this.confirmSelectedTime();
        }
        else {
            this.cancelSelection();
        }
    }
    /**
     * Change values to last confirm select time
     */
    cancelSelection() {
        this._hour = this._selectedHour;
        this._minute = this._selectedMinute;
        this._period = this._selectedPeriod;
        this.isOpen = false;
    }
    /**
     * Set new values of time and emit new event with the formatted timer
     */
    confirmSelectedTime() {
        this._selectedHour = this.hour;
        this._selectedMinute = this.minute;
        this._selectedPeriod = this.period;
        // format string to emit selected time
        let formated;
        if (this.format === '12') {
            formated = `${this.hour}:${this.minute} ${this.period}`;
        }
        else {
            let hour = this.hour;
            if (this.period === 'pm') {
                hour = `${parseInt(hour, 10) + 12}`;
            }
            formated = `${hour}:${this.minute}`;
        }
        this.selected.emit(formated);
        this.isOpen = false;
    }
}
MccTimerPickerComponent.decorators = [
    { type: Component, args: [{
                selector: 'mcc-timer-picker',
                template: "<!-- show component inside overlay -->\n<ng-container *ngIf=\"connected\">\n  <ng-template cdkConnectedOverlay cdkConnectedOverlayHasBackdrop cdkConnectedOverlayBackdropClass=\"mcc-timer-picker-backdrop\" [cdkConnectedOverlayOrigin]=\"trigger\"\n    [cdkConnectedOverlayOpen]=\"isOpen\" (backdropClick)=\"backdropClick()\">\n\n    <ng-template [cdkPortalOutlet]=\"panelComponent\"></ng-template>\n\n  </ng-template>\n</ng-container>\n\n<ng-template *ngIf=\"!connected\" [cdkPortalOutlet]=\"panelComponent\"></ng-template>\n\n<ng-template cdkPortal #panelComponent=\"cdkPortal\">\n  <div class=\"mcc-timer-picker-overlay mat-elevation-z4\" role=\"dialog\" aria-label=\"Timer picker\">\n\n    <div class=\"mcc-timer-picker-header mat-primary\">\n      <div class=\"mcc-timer-picker-header-clock\">\n        <button mat-icon-button class=\"mcc-timer-picker-hours\" [ngClass]=\"{ 'mcc-active': focus === 'hour' }\"\n                (click)=\"focus = 'hour'\">\n         <span><span *ngIf=\"needsSpacer\">&nbsp;&nbsp;</span>{{ hour }}</span>\n        </button>\n        <span class=\"mcc-timer-picker-separator\">:</span>\n        <button mat-icon-button class=\"mcc-timer-picker-minutes\" [ngClass]=\"{ 'mcc-active': focus === 'min' }\"\n                (click)=\"focus = 'min'\">\n          <span>{{ minute }}</span>\n        </button>\n\n        <div class=\"mcc-timer-picker-am-pm\" *ngIf=\"format === '12'\">\n          <button mat-icon-button [ngClass]=\"{ 'mcc-active': period === 'am' }\" (click)=\"changePeriod('am')\">\n            AM\n          </button>\n          <button mat-icon-button [ngClass]=\"{ 'mcc-active': period === 'pm' }\" (click)=\"changePeriod('pm')\">\n            PM\n          </button>\n        </div>\n      </div>\n    </div>\n\n    <div class=\"mcc-timer-picker-content\">\n\n      <div class=\"mcc-timer-picker-clock\">\n        <div class=\"mcc-timer-picker-dot\"></div>\n        <div class=\"mcc-timer-picker-line\" [ngClass]=\"getSelectedClass()\"></div>\n        <ng-container *ngFor=\"let option of clock$ | async;let i = index;\">\n          <button mat-icon-button class=\"mcc-timer-picker-option\" [ngClass]=\"{\n          'mcc-timer-picker-option-selected': focus === 'hour' && hour == option || focus === 'min' && minute === option }\"\n            [id]=\"'option-'+i\" (click)=\"select(option)\" [disabled]=\"isOptionDisabled(option)\">\n            {{ option }}\n          </button>\n\n        </ng-container>\n\n        <ng-container *ngIf=\"format === '24' && focus === 'hour'\">\n          <ng-container *ngFor=\"let option of hours24; let i = index;\">\n            <button mat-icon-button class=\"mcc-timer-picker-option hours\" [ngClass]=\"{\n              'mcc-timer-picker-option-selected': hour == option}\"\n                    [id]=\"'option-hour-'+i\" (click)=\"select(option)\" [disabled]=\"isOptionDisabled(option)\">\n              {{ option }}\n            </button>\n          </ng-container>\n        </ng-container>\n      </div>\n\n    </div>\n\n    <div *ngIf=\"!hideButtons\" class=\"mcc-timer-picker-actions\">\n      <div style=\"flex: 1 auto;\"></div>\n      <button mat-button role=\"button\" aria-label=\"Cancel\" (click)=\"cancelSelection()\">\n        {{ btnCancel }}\n      </button>\n      <button mat-button role=\"button\" aria-label=\"Confirm\" (click)=\"confirmSelectedTime()\">\n        {{ btnConfirm }}\n      </button>\n    </div>\n\n  </div>\n</ng-template>\n",
                preserveWhitespaces: false,
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [".mcc-timer-picker-overlay{background-color:#fff;border-radius:4px;font-family:Roboto,sans-serif;width:290px}.mcc-timer-picker-header{background:#00796b;border-top-left-radius:4px;border-top-right-radius:4px;box-sizing:border-box;display:flex;justify-content:center;padding:4px 0;width:100%}.mcc-timer-picker-header .mcc-timer-picker-header-clock{align-items:center;display:flex}.mcc-timer-picker-hours,.mcc-timer-picker-minutes{color:hsla(0,0%,100%,.3);font-size:50px;font-weight:400;height:63px;width:63px}.mcc-timer-picker-hours.mcc-standalone,.mcc-timer-picker-minutes.mcc-standalone{text-align:right}.mcc-timer-picker-hours.mcc-active,.mcc-timer-picker-minutes.mcc-active{color:#fff}.mcc-timer-picker-24hours{color:hsla(0,0%,100%,.3);font-size:50px;font-weight:400;height:63px;width:63px}.mcc-timer-picker-24hours.mcc-standalone{text-align:right}.mcc-timer-picker-24hours.mcc-active{color:#fff}.mcc-timer-picker-separator{color:hsla(0,0%,100%,.3);font-size:43px;font-weight:400}.mcc-timer-picker-am-pm{display:flex;flex-direction:column}.mcc-timer-picker-am-pm button{color:hsla(0,0%,100%,.3);font-size:12px;font-weight:400;height:25px;line-height:25px;width:25px}.mcc-timer-picker-am-pm button.mcc-active{color:#fff}.mcc-timer-picker-content{height:260px;padding:15px 0 5px;width:inherit}.mcc-timer-picker-clock{background-color:#eee;border-radius:50%;height:250px;margin:0 auto;position:relative;text-align:center;width:250px}.mcc-timer-picker-clock .mcc-timer-picker-dot{background-color:#00796b;border-radius:50%;height:10px;left:120px;position:absolute;top:120px;width:10px}.mcc-timer-picker-clock .mcc-timer-picker-line{background-color:#00796b;height:100px;left:124px;position:absolute;top:10%;transform-origin:50% 100%;width:2px}.mcc-timer-picker-clock .mcc-timer-picker-line.hour-format-24{height:75px;top:20%}.mcc-timer-picker-clock .mcc-timer-picker-line.selected-index-0{transform:rotate(0deg)}.mcc-timer-picker-clock .mcc-timer-picker-line.selected-index-1{transform:rotate(327deg)}.mcc-timer-picker-clock .mcc-timer-picker-line.selected-index-2{transform:rotate(30deg)}.mcc-timer-picker-clock .mcc-timer-picker-line.selected-index-3{transform:rotate(300deg)}.mcc-timer-picker-clock .mcc-timer-picker-line.selected-index-4{transform:rotate(60deg)}.mcc-timer-picker-clock .mcc-timer-picker-line.selected-index-5{transform:rotate(270deg)}.mcc-timer-picker-clock .mcc-timer-picker-line.selected-index-6{transform:rotate(90deg)}.mcc-timer-picker-clock .mcc-timer-picker-line.selected-index-7{transform:rotate(240deg)}.mcc-timer-picker-clock .mcc-timer-picker-line.selected-index-8{transform:rotate(120deg)}.mcc-timer-picker-clock .mcc-timer-picker-line.selected-index-9{transform:rotate(210deg)}.mcc-timer-picker-clock .mcc-timer-picker-line.selected-index-10{transform:rotate(150deg)}.mcc-timer-picker-clock .mcc-timer-picker-line.selected-index-11{transform:rotate(180deg)}.mcc-timer-picker-clock .mcc-timer-picker-option{position:absolute}.mcc-timer-picker-clock .mcc-timer-picker-option.hours.mat-icon-button{font-size:70%;height:26px;line-height:10px;width:26px}.mcc-timer-picker-clock .mcc-timer-picker-option.hours#option-hour-0{left:112px;top:45px}.mcc-timer-picker-clock .mcc-timer-picker-option.hours#option-hour-1{left:80px;top:57px}.mcc-timer-picker-clock .mcc-timer-picker-option.hours#option-hour-2{right:80px;top:57px}.mcc-timer-picker-clock .mcc-timer-picker-option.hours#option-hour-3{left:57px;top:81px}.mcc-timer-picker-clock .mcc-timer-picker-option.hours#option-hour-4{right:57px;top:81px}.mcc-timer-picker-clock .mcc-timer-picker-option.hours#option-hour-5{left:45px;top:112px}.mcc-timer-picker-clock .mcc-timer-picker-option.hours#option-hour-6{right:45px;top:112px}.mcc-timer-picker-clock .mcc-timer-picker-option.hours#option-hour-7{bottom:81px;left:57px}.mcc-timer-picker-clock .mcc-timer-picker-option.hours#option-hour-8{bottom:81px;right:57px}.mcc-timer-picker-clock .mcc-timer-picker-option.hours#option-hour-9{bottom:57px;left:80px}.mcc-timer-picker-clock .mcc-timer-picker-option.hours#option-hour-10{bottom:57px;right:80px}.mcc-timer-picker-clock .mcc-timer-picker-option.hours#option-hour-11{bottom:45px;left:112px}.mcc-timer-picker-clock .mcc-timer-picker-option.mcc-timer-picker-option-selected{background-color:#00796b;color:#fff}.mcc-timer-picker-clock .mcc-timer-picker-option#option-0{left:105px;top:0}.mcc-timer-picker-clock .mcc-timer-picker-option#option-1{left:52px;top:15px}.mcc-timer-picker-clock .mcc-timer-picker-option#option-2{right:52px;top:16px}.mcc-timer-picker-clock .mcc-timer-picker-option#option-3{left:16px;top:52px}.mcc-timer-picker-clock .mcc-timer-picker-option#option-4{right:16px;top:52px}.mcc-timer-picker-clock .mcc-timer-picker-option#option-5{left:0;top:105px}.mcc-timer-picker-clock .mcc-timer-picker-option#option-6{right:0;top:105px}.mcc-timer-picker-clock .mcc-timer-picker-option#option-7{bottom:52px;left:16px}.mcc-timer-picker-clock .mcc-timer-picker-option#option-8{bottom:52px;right:16px}.mcc-timer-picker-clock .mcc-timer-picker-option#option-9{bottom:16px;left:52px}.mcc-timer-picker-clock .mcc-timer-picker-option#option-10{bottom:16px;right:52px}.mcc-timer-picker-clock .mcc-timer-picker-option#option-11{bottom:0;left:105px}.mcc-timer-picker-actions{display:flex;padding:4px}.mcc-timer-picker-actions button{text-transform:uppercase}mat-icon-button{height:30px;line-height:30px;width:30px}"]
            },] }
];
MccTimerPickerComponent.ctorParameters = () => [
    { type: ChangeDetectorRef }
];
MccTimerPickerComponent.propDecorators = {
    hideButtons: [{ type: Input }],
    format: [{ type: Input, args: ['mccTimerPickerFormat',] }],
    min: [{ type: Input, args: ['mccTimerPickerMin',] }],
    max: [{ type: Input, args: ['mccTimerPickerMax',] }],
    btnCancel: [{ type: Input }],
    btnConfirm: [{ type: Input }],
    selected: [{ type: Output }]
};

class MccTimerPickerOriginDirective {
    /**
     * Reference to the element on which the directive is applied.
     */
    constructor(elementRef, renderer) {
        this.elementRef = elementRef;
        this.renderer = renderer;
        /**
         * Emit changes from the origin
         */
        this.change = new BehaviorSubject('');
        /**
         * Emit changes from the origin
         */
        this.hasFocus = new EventEmitter();
    }
    onFocus() {
        this.hasFocus.emit();
    }
    /**
     * This method will be called by the forms API to write to the view when
     * programmatic (model -> view) changes are requested.
     */
    writeValue(time) {
        this.renderer.setProperty(this.elementRef.nativeElement, 'value', time);
        this.change.next(time);
        if (this.propagateChanges) {
            this.propagateChanges(time);
        }
    }
    /**
     * This method will be called by the time picker
     */
    writeValueFromTimerPicker(time) {
        this.renderer.setProperty(this.elementRef.nativeElement, 'value', time);
        this.propagateChanges(time);
    }
    /**
     * This method will be called from origin whe key is up
     */
    writeValueFromKeyup(time) {
        this.change.next(time);
        this.propagateChanges(time);
    }
    /**
     * This is called by the forms API on initialization so it can update the
     * form model when values propagate from the view (view -> model).
     * @param fn any
     */
    registerOnChange(fn) {
        this.propagateChanges = fn;
    }
    /**
     * This is called by the forms API on initialization so it can update the form model on blur
     * @param fn any
     */
    registerOnTouched(fn) { }
    /**
     * called by the forms API when the control status changes to or from "DISABLED"
     * @param isDisabled boolean
     */
    setDisabledState(isDisabled) {
        this.renderer.setProperty(this.elementRef.nativeElement, 'disabled', isDisabled);
    }
}
MccTimerPickerOriginDirective.decorators = [
    { type: Directive, args: [{
                selector: '[mccTimerPickerOrigin], [mcc-timer-picker-origin]',
                exportAs: 'mccTimerPickerOrigin',
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => MccTimerPickerOriginDirective),
                        multi: true,
                    },
                ],
            },] }
];
MccTimerPickerOriginDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Renderer2 }
];
MccTimerPickerOriginDirective.propDecorators = {
    change: [{ type: Output }],
    hasFocus: [{ type: Output }],
    onFocus: [{ type: HostListener, args: ['focus',] }]
};
class MccConnectedTimerPickerDirective {
    constructor(timerPicker, changeDetectorRef) {
        this.timerPicker = timerPicker;
        this.changeDetectorRef = changeDetectorRef;
        this.timerPicker.connected = true;
    }
    ngAfterViewInit() {
        if (!this._timerPickerSub) {
            this.timerPicker.trigger = this.origin;
            this._attachTimerPicker();
        }
    }
    ngOnDestroy() {
        if (this._originFocus && !this._originFocus.closed) {
            this._originFocus.unsubscribe();
        }
        if (this._timerPickerSub && !this._timerPickerSub.closed) {
            this._timerPickerSub.unsubscribe();
        }
    }
    /**
     * Attach the timer picker to origin element (input)
     */
    _attachTimerPicker() {
        this._originFocus = this.origin.hasFocus.subscribe(() => {
            this.timerPicker.focus = 'hour';
            this.timerPicker.isOpen = true;
            this.changeDetectorRef.detectChanges();
        });
        this._timerPickerSub = this.timerPicker.selected.subscribe(value => this.origin.writeValueFromTimerPicker(value));
    }
}
MccConnectedTimerPickerDirective.decorators = [
    { type: Directive, args: [{
                selector: '[mccConnectedTimerPicker], [mcc-connected-timer-picker]',
                exportAs: 'mccConnectedTimerPicker',
            },] }
];
MccConnectedTimerPickerDirective.ctorParameters = () => [
    { type: MccTimerPickerComponent },
    { type: ChangeDetectorRef }
];
MccConnectedTimerPickerDirective.propDecorators = {
    origin: [{ type: Input, args: ['mccConnectedTimerPickerOrigin',] }]
};

class MccTimerPickerModule {
}
MccTimerPickerModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule, PortalModule, OverlayModule, MatButtonModule],
                declarations: [
                    MccTimerPickerComponent,
                    MccTimerPickerOriginDirective,
                    MccConnectedTimerPickerDirective,
                ],
                exports: [
                    MccTimerPickerComponent,
                    MccTimerPickerOriginDirective,
                    MccConnectedTimerPickerDirective,
                ],
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { HOURS, HOURS24, MINUTES, MccConnectedTimerPickerDirective, MccTimerPickerComponent, MccTimerPickerModule, MccTimerPickerOriginDirective };
//# sourceMappingURL=material-community-components-timer-picker.js.map
